#! /bin/bash


# test tftp and nfsroot export
# note that this script can not test ip-based permissions
# and that even if this script is successful, protection schemes such as tcp wrappers could prevent 
# ka from accessing these services

# note also that the NFS test can only be done if the tftp test is OK, since
# the parameters for the nfsroot export are extracted from the pxelinux template file


tftp_server=$1

if test -f /etc/ka/ka.conf; then
	. /etc/ka/ka.conf
	. /etc/ka/ka.funcs
fi

if test -z "$tftp_server"; then
	echo You must supply the tftp server IP address
	exit 1
fi



nfs_server=""
nfsroot_dir=""

std_tftp_get_file()
{
	remotef=$1
	localf=$2
	echo tftp getting file $remotef from server $tftp_server
	terr=`echo get $remotef $localf | tftp $tftp_server 2>&1`
	err=`echo $terr | grep Received`
	if [ -z "$err" ]; then
		echo tftp error: could not get/put file 
		echo Error was : +++ $terr +++
		return 1
	fi
	return 0
}



tftp_test()
{
	tmp=/tmp/test_services$$
	if std_tftp_get_file "ka/pxelinux.cfg/template" $tmp; then
		echo OK
		lab=`grep LABEL $tmp`
		nfsinfo=`grep -v ^# $tmp | grep APPEND | head -1 | sed 's/.*nfsroot=\([^ ]*\).*/\1/'`
		rm -f $tmp
		if [ "$lab" ]; then
			echo template file seems OK

			nfs_server=`echo $nfsinfo | cut -d : -f 1`
			nfsroot_dir=`echo $nfsinfo | cut -d : -f 2-`
			
			echo Guessed NFS server is $nfs_server
			echo Guessed nfsroot dir is $nfsroot_dir

			return 0
		else
			echo this does not look like a template file, tftp problem ?
			return 1
		fi
	else
		echo Failed
		return 1
	fi
}


nfsroot_files_tests()
{
	Ntftp_server=`cat ka/tftpserver`
	if [ x$Ntftp_server != x$tftp_server ]; then
		echo 'warning : tftp server from ka.conf and (nfsroot)/ka/tftpserver do not match'
		echo $tftp_server "!=" $Ntftp_server
	else
		echo '(nfsroot)/ka/tftpserver seems ok'
	fi
}

nfsroot_mount_test()
{
	tmpdir=/tmp/test_services$$

	echo Testing nfsroot mounting

	mkdir $tmpdir

	echo Trying to mount $nfs_server:$nfsroot_dir as $tmpdir
	if ! mount -o ro -t nfs $nfs_server:$nfsroot_dir $tmpdir; then
		echo mount failed
		return 1
	fi

	echo mount OK
	
	mount | grep $nfs_server:$nfsroot_dir 
	
	if ! test -f $tmpdir/ka/install.sh; then
		echo could not access /ka/install.sh on nfsroot filesystem
		umount $tmpdir
		return 1
	fi
	
	echo Filesystem seems OK

	( cd $tmpdir && nfsroot_files_tests )
	
	umount $tmpdir
	return 0
}

	
nfsroot_rpcinfo_test()
{
	echo Running rpcinfo
	info=`/usr/sbin/rpcinfo -p $nfs_server`
	if [ $? -ne 0 ]; then
		echo Could not run rpcinfo
		return 1
	fi
	
	if echo "$info" | grep -q portmapper; then
		echo Portmapper OK
	else
		echo Portmapper missing ?
		return 1
	fi
	
	if echo "$info" | grep -q mountd; then
		echo Mountd OK
	else
		echo Mountd missing ?
		return 1
	fi
	
	
	if echo "$info" | grep -q nfs; then
		echo Nfs OK
	else
		echo Nfs missing ?
		return 1
	fi
	
	
	return 0
}
	
	

if ! tftp_test ; then exit; fi
if ! nfsroot_rpcinfo_test ; then exit; fi

if [ $UID -ne 0 ]; then
	echo nfsroot mount test can only be done as root.. SKIPPED.
else
	nfsroot_mount_test
fi
